##
# PloneSVNaccess<br>
#  Providing read and write access to subversion.<br>
#<br>
# PloneSVNaccess.py<br>
#  The Product.<br>
#<br>
# Author: Christian Koller 'admin@iecw.net'.<br>
# Extended by: Thomas Mller<br>
#<br>
# Released under GNU GPL v2 or later.
##

from Products.Archetypes.public import *
from Products.PloneSVNaccess.config import PROJECTNAME
from Products.CMFCore import CMFCorePermissions
from Products.validation import V_REQUIRED
from AccessControl import ClassSecurityInfo
import pysvn

from zLOG import LOG, INFO

from Products.PloneSVNaccess.config import PROTOCOLS, PROJECTNAME, view_permission, edit_permission, write_permission

##
# Plone Subversion access<br>
# The main class, setting the schema, the content_icon, the actions (Tabs) and a lot of protected methods 
# for the page templates
class PloneSVNaccess(BaseContent):
	""" Plone Subversion access
		The main class, setting the schema, the content_icon, the actions (Tabs) and a lot of protected methods 
		for the page templates
	"""
	
	meta_type = PROJECTNAME
	
	security  = ClassSecurityInfo()
	
	##
	#  The schema includes five variables, the access_mode, the repository_path, the username and password for the
	#  repository and the webdav_path
	#

	schema = BaseSchema + Schema((

		StringField('access_mode',
			searchable=0,
			required=0,
			validators=('isAllowedSVNurl'),
			vocabulary = PROTOCOLS,
			widget=SelectionWidget(
				label='Access mode',
				label_msgid='access_mode',
				i18n_domain='PloneSVNaccess',
				format='flex',
			),
		),
		StringField('repository_path',
			searchable=0,
			required=1,
			widget=StringWidget(
				label='Repository URL/PATH',
				label_msgid='repository_path',
				size='70',
				i18n_domain='PloneSVNaccess',
			),
		),
		StringField('username',
			searchable=0,
			required=0,
			widget=StringWidget(
				label='Username',
				label_msgid='username',
				description='A Username to log in through svn:// , leave empty for anonymous access.',
				description_msgid='username_description',
				size='15',
				i18n_domain='PloneSVNaccess',
			),
		),
		StringField('password',
			searchable=0,
			required=0,
			widget=PasswordWidget(
				label='Password',
				label_msgid='password',
				description='The password to log in through svn:// , leave empty for anonymous access.',
				description_msgid='password_description',
				size='15',
				i18n_domain='PloneSVNaccess',
			),
		),
		StringField('webdav_path',
			searchable=0,
			required=0,
			widget=StringWidget(
				label='WebDAV URL',
				label_msgid='webdav_url',
				description='The WebDAV URL used by users. e.g.: http://user:password@www.example.com/svn/repository',
				description_msgid='webdav_url_description',
				size='90',
				i18n_domain='PloneSVNaccess',
			),
		),
		StringField(
			name='trackerUrl',
			widget=StringWidget(
				label="URL to Bug-Tracker",
				description="""Please enter the URL to the related bug tracker. e.g.: /poi/%(bug)s or https://bugzilla.mozilla.org/show_bug.cgi?id=%(bug)s""",
				label_msgid='trackerurl',
				description_msgid='trackerurl_description',
				i18n_domain='PloneSVNaccess',
				size = '90',
			),
			required=False,
		),
		BooleanField('revisionlink_detection',
			required=0,
			default=False,
			widget=BooleanWidget(
				label='Revision Link Creation',
				label_msgid='revisionlink_detection',
				description="""Revisions will be detected and will appear as links within PloneSVNAccess.""",
				description_msgid='revisionlink_detection_description',
				i18n_domain='PloneSVNaccess',
			)
		),

	))
	
	##
	# Set the icon for the PloneSVNaccess
	content_icon = 'PloneSVNaccess.png'



	##
	#  Redefine the view tab (action) and define the view_log and local_roles for PloneSVNaccess
	#
	actions = (
		{
			'id': 'view',
			'name': 'View',
			'action': 'string:${object_url}/svn_view',
			'permissions': (view_permission,),
			'category':'object',
		},
		{
			'id': 'view_log',
			'name': 'Log',
			'action': 'string:${object_url}/svn_view_log',
			'permissions': (view_permission,),
			'category':'object',
		},
		{
			'id': 'local_roles',
			'name': 'Sharing',
			'action': 'string:${object_url}/folder_localrole_form',
			'permissions' : (edit_permission, ),
			'category' : 'object'
		},
	)



	security.declareProtected(view_permission, "getDiff")
	##
	# Return File Diff<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getDiff-method">PloneSVNaccessTool.py</a>
	#
	def getDiff(self, svn_path, username, password, curSVNFile, curRevNo, otherRevNo):
		""" Return File Diff """
		return self.PloneSVNaccessTool.getDiff(svn_path, username, password, curSVNFile, curRevNo, otherRevNo)



	security.declareProtected(view_permission, "getSingleFile")
	##
	# Get a single file<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getSingleFile-method">PloneSVNaccessTool.py</a>
	#
	def getSingleFile(self, svn_path, username, password, curSVNFile, revNo, errorType, forDownload):
		"""
			Get a single file
			errorType=0 raise all errors
			errorType=1 reply with False if file is not available with this revision number
			forDownload = True -> set PloneSVNaccessVersioning
		"""
		return self.PloneSVNaccessTool.getSingleFile(svn_path, username, password, curSVNFile, revNo, errorType, forDownload)



	security.declareProtected(view_permission, "getFile")
	##
	# Get the file info<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getFile-method">PloneSVNaccessTool.py</a>
	#
	def getFile(self, svn_path, username, password, curSVNFile, revNo):
		""" Get the file info """
		return self.PloneSVNaccessTool.getFile(svn_path, username, password, curSVNFile, revNo)



	security.declareProtected(view_permission, "getLog")
	##
	# Return Log info<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getLog-method">PloneSVNaccessTool.py</a>
	#
	def getLog(self, svn_path, username, password, folder, curRevNo, startRevNo = None):
		""" Return Log info """
		return self.PloneSVNaccessTool.getLog(svn_path, username, password, folder, curRevNo, startRevNo, self.getTrackerUrl(), self.getRevisionlink_detection())



	security.declareProtected(view_permission, "getDir")
	##
	# Get the folder listing<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getDir-method">PloneSVNaccessTool.py</a>
	#
	def getDir(self, svn_path, username, password, folder, revNo):
		""" Get the folder listing """
		return self.PloneSVNaccessTool.getDir(svn_path, username, password, folder, revNo)



	security.declareProtected(view_permission, "getRevNos")
	##
	# Returns Revision Numbers<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getRevNos-method">PloneSVNaccessTool.py</a>
	#
	def getRevNos(self, curRevNo, maxRevNo, svn_path, username, password, folderOrFile):
		""" Returns Revision Numbers """
		return self.PloneSVNaccessTool.getRevNos(curRevNo, maxRevNo, svn_path, username, password, folderOrFile)



	security.declareProtected(view_permission, "getMaxRevNo")
	##
	# Return the RevNo of head<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getMaxRevNo-method">PloneSVNaccessTool.py</a>
	#
	def getMaxRevNo(self, svn_path, username, password):
		""" Return the RevNo of head """
		return self.PloneSVNaccessTool.getMaxRevNo(svn_path, username, password)



	security.declareProtected(view_permission, "getFileContent")
	##
	# Get File Content<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getSingleFile-method">PloneSVNaccessTool.py</a>
	#
	def getFileContent(self, REQUEST, RESPONSE):
		""" Export the file content """
		theFilename = REQUEST.form['curSVNFile']
		curRevNo = REQUEST.form['curRevNo']
		forDownload = not REQUEST.form.has_key('forView')
		
		theFile = self.PloneSVNaccessTool.getSingleFile(self.access_mode + self.repository_path, self.username, self.password, theFilename, curRevNo, errorType=0, forDownload = forDownload)
		
		RESPONSE.setHeader('Content-Disposition',
                           'attachment; filename="' + theFilename.split('/').pop() + '"')
		RESPONSE.setHeader('Content-Type', theFile.mime)
		try:
			RESPONSE.setHeader('Content-Length', theFile.info.size)
		except:
			pass
		RESPONSE.write(theFile.file_content)
		
		#Debug:
		#RESPONSE.write(str(REQUEST.form))



	security.declareProtected(view_permission, "getFolderContent")
	##
	# Get Folder as TAR or ZIP<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.getFolderAsTarOrZip-method">PloneSVNaccessTool.py</a>
	#
	def getFolderContent(self, REQUEST, RESPONSE):
		""" Get Folder as TAR or ZIP """
		theFolder = REQUEST.form['curSVNFolder']
		fileFormat = REQUEST.form['fileFormat'].lower()
		exportType = REQUEST.form['exportType'].lower()

		##
		# Get the max Revision number.
		maxRevNo = self.PloneSVNaccessTool.getMaxRevNo(self.access_mode + self.repository_path, self.username, self.password)
		
		##
		# Get the handle to the Tar or Zip file
		theFile = self.PloneSVNaccessTool.getFolderAsTarOrZip(self.access_mode + self.repository_path, self.username, self.password, theFolder, fileFormat, exportType, maxRevNo)


		theTarOrZipFilename = self.repository_path.replace("/", "_") + theFolder.replace("/", "_") + '_rev' + str(maxRevNo) + "_exportType" + str(exportType)
		
		if fileFormat == "tar":		
			RESPONSE.setHeader('Content-Type', "application/application/x-compressed-tar")
			RESPONSE.setHeader('Content-Disposition',
				'attachment; filename=%s.tar.gz' % theTarOrZipFilename)
		else:
			RESPONSE.setHeader('Content-Type', "application/application/zip")
			RESPONSE.setHeader('Content-Disposition',
				'attachment; filename=%s.zip' % theTarOrZipFilename)
		
		# Read the file from the handle
		theFile.seek(0)
		while True:
			theFileContent = theFile.read()
			if not theFileContent:
				break
			RESPONSE.write(theFileContent)
		theFile.close()



	security.declareProtected(write_permission, "commitSVNfile")
	##
	# Commit file to svn.<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.commit_svnFile-method">PloneSVNaccessTool.py</a>
	#
	def commitSVNfile(self, REQUEST, RESPONSE):
		""" Commit file to svn """
		curFileContent = REQUEST.get('curFileContent', None)
		curFile = REQUEST.get('curFile', None)
		curMessage = REQUEST.get('curMessage', None)
		curSVNFolder = REQUEST.get('curSVNFolder', None)
		curSVNFile = REQUEST.get('curSVNFile', None)
		curRevNo = REQUEST.get('curRevNo', None)
		
		if curFile is not None and curFile.filename:
			# We get a file - use it
			# Try to commit the file
			responseMsg = self.PloneSVNaccessTool.commit_svnFile(self.access_mode + self.repository_path, self.username, self.password, curSVNFolder, curSVNFile, curMessage, curFile = curFile)
			
			if responseMsg[:8] == "SVNMERGE":
				# The file needs a merge
				session  = REQUEST.SESSION
				session.set('fileContent' + curSVNFile, responseMsg[8:])				
				return REQUEST.RESPONSE.redirect('svn_merge?curSVNFile=' + curSVNFile + '&curSVNFolder=' + curSVNFolder + '&curRevNo=' + curRevNo + '&curMessage=' + curMessage)
			else:
				# Just reply with responseMsg
				return REQUEST.RESPONSE.redirect('svn_view_file?curSVNFile=' + curSVNFile + '&curSVNFolder=' + curSVNFolder + '&curRevNo=&infoMessage=' + responseMsg)
		
		elif curFileContent is not None:
			# We got the file content - use it
			# Commit the file
			responseMsg = self.PloneSVNaccessTool.commit_svnFile(self.access_mode + self.repository_path, self.username, self.password, curSVNFolder, curSVNFile, curMessage, curFileContent = curFileContent)
			# Just reply with responseMsg
			return REQUEST.RESPONSE.redirect('svn_view_file?curSVNFile=' + curSVNFile + '&curSVNFolder=' + curSVNFolder + '&curRevNo=&infoMessage=' + responseMsg)
			
		else:
			# No file - send a error message
			return REQUEST.RESPONSE.redirect('svn_view_file?curSVNFile=' + curSVNFile + '&curSVNFolder=' + curSVNFolder + '&curRevNo=' + curRevNo + '&infoMessage=Please select a file!')		



	security.declareProtected(write_permission, "uploadSVNfile")
	##
	# Upload file to svn<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.upload_svnFile-method">PloneSVNaccessTool.py</a>
	#
	def uploadSVNfile(self, REQUEST, RESPONSE):
		""" Upload file to svn """
		curFile = REQUEST.get('curFile', None)
		curMessage = REQUEST.get('curMessage', None)
		curSVNFolder = REQUEST.get('curSVNFolder', None)
		curRevNo = REQUEST.get('curRevNo', None)
		#LOG('PloneSVNaccessTool', 0, 'uploadSVNfile:', curFile.filename)

		svn_path = self.access_mode + self.repository_path
		theNewFolder = curSVNFolder + '/' + curFile.filename		
		theFullPath = svn_path.encode('utf-8') + theNewFolder
		
		if curFile is not None and curFile.filename:
			responseMsg = self.PloneSVNaccessTool.upload_svnFile(self.username, self.password, theFullPath, curFile, curMessage)			
			return REQUEST.RESPONSE.redirect('svn_view?curSVNFolder=' + curSVNFolder + '&curRevNo=&infoMessage=' + responseMsg)
		else:
			return REQUEST.RESPONSE.redirect('svn_view?curSVNFolder=' + curSVNFolder + '&curRevNo=' + curRevNo + '&infoMessage=Please select a file!')		



	security.declareProtected(write_permission, "deleteFileOrFolder")
	##
	# Delete File or Folder<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.deleteFileOrFolder-method">PloneSVNaccessTool.py</a>
	#
	def deleteFileOrFolder(self, REQUEST, RESPONSE):
		""" Delete File or Folder """
		##
		# What to delete?
		curSVNFile = REQUEST.get('curSVNFile', None)
		curSVNFolder = REQUEST.get('curSVNFolder', None)
		curRevNo = REQUEST.get('curRevNo', None)
		curMessage = REQUEST.get('curMessage', None)
		responseMsg = self.PloneSVNaccessTool.deleteFileOrFolder(self.access_mode + self.repository_path, self.username, self.password, curSVNFile, curMessage)			
		return REQUEST.RESPONSE.redirect('svn_view?curSVNFolder=' + curSVNFolder + '&curRevNo=&infoMessage=' + responseMsg)		



	security.declareProtected(write_permission, "newFolder")
	##
	# Create a new Folder<br>
	#
	# Calls: <a href="pythondoc-PloneSVNaccessTool.html#PloneSVNaccessTool.PloneSVNaccessTool.createFolder-method">PloneSVNaccessTool.py</a>
	#
	def newFolder(self, REQUEST, RESPONSE):
		""" Create a new Folder """
		##
		# What to create?
		theFolderName = REQUEST.get('theFolderName', None)
		curSVNFolder = REQUEST.get('curSVNFolder', None)
		curRevNo = REQUEST.get('curRevNo', None)
		curMessage = REQUEST.get('curMessage', None)
		
		svn_path = self.access_mode + self.repository_path
		theNewFolder = curSVNFolder + '/' + theFolderName		
		theFullPath = svn_path.encode('utf-8') + theNewFolder
		
		responseMsg = self.PloneSVNaccessTool.createFolder(self.username, self.password, theNewFolder, curMessage, theFullPath)
		return REQUEST.RESPONSE.redirect('svn_view?curSVNFolder=' + curSVNFolder + '&curRevNo=&infoMessage=' + responseMsg)		
	
	
# Register the new type within Zope
registerType(PloneSVNaccess,PROJECTNAME)


